unit ntddk;

interface

uses
  Windows;

////////////////////////////////////////////////////////////////////////////////
// for older Delphi versions

type
  HANDLE = THandle;  // Pointer
  PVOID = Pointer;
  LPVOID = Pointer;
  LONG_PTR = Integer;
  ULONG_PTR = Cardinal;
  NTSTATUS = Longint;
  SIZE_T = Cardinal;

const
  IMAGE_NUMBEROF_DIRECTORY_ENTRIES = 16;
  IMAGE_SIZEOF_SHORT_NAME = 8;

type
  PImageDosHeader = ^TImageDosHeader;
  TImageDosHeader = packed record      // DOS .EXE header
    e_magic   : Word;                  // Magic number
    e_cblp    : Word;                  // Bytes on last page of file
    e_cp      : Word;                  // Pages in file
    e_crlc    : Word;                  // Relocations
    e_cparhdr : Word;                  // Size of header in paragraphs
    e_minalloc: Word;                  // Minimum extra paragraphs needed
    e_maxalloc: Word;                  // Maximum extra paragraphs needed
    e_ss      : Word;                  // Initial (relative) SS value
    e_sp      : Word;                  // Initial SP value
    e_csum    : Word;                  // Checksum
    e_ip      : Word;                  // Initial IP value
    e_cs      : Word;                  // Initial (relative) CS value
    e_lfarlc  : Word;                  // File address of relocation table
    e_ovno    : Word;                  // Overlay number
    e_res     : array [0..3] of Word;  // Reserved words
    e_oemid   : Word;                  // OEM identifier (for e_oeminfo)
    e_oeminfo : Word;                  // OEM information; e_oemid specific
    e_res2    : array [0..9] of Word;  // Reserved words
    _lfanew   : Longint;               // File address of new exe header
  end;

  PImageFileHeader = ^TImageFileHeader;
  TImageFileHeader = packed record
    Machine             : Word;
    NumberOfSections    : Word;
    TimeDateStamp       : DWORD;
    PointerToSymbolTable: DWORD;
    NumberOfSymbols     : DWORD;
    SizeOfOptionalHeader: Word;
    Characteristics     : Word;
  end;

  PImageDataDirectory = ^TImageDataDirectory;
  TImageDataDirectory = record
    VirtualAddress: DWORD;
    Size          : DWORD;
  end;

  PImageOptionalHeader = ^TImageOptionalHeader;
  TImageOptionalHeader = packed record
    Magic                      : Word;
    MajorLinkerVersion         : Byte;
    MinorLinkerVersion         : Byte;
    SizeOfCode                 : DWORD;
    SizeOfInitializedData      : DWORD;
    SizeOfUninitializedData    : DWORD;
    AddressOfEntryPoint        : DWORD;
    BaseOfCode                 : DWORD;
    BaseOfData                 : DWORD;
    ImageBase                  : DWORD;
    SectionAlignment           : DWORD;
    FileAlignment              : DWORD;
    MajorOperatingSystemVersion: Word;
    MinorOperatingSystemVersion: Word;
    MajorImageVersion          : Word;
    MinorImageVersion          : Word;
    MajorSubsystemVersion      : Word;
    MinorSubsystemVersion      : Word;
    Win32VersionValue          : DWORD;
    SizeOfImage                : DWORD;
    SizeOfHeaders              : DWORD;
    CheckSum                   : DWORD;
    Subsystem                  : Word;
    DllCharacteristics         : Word;
    SizeOfStackReserve         : DWORD;
    SizeOfStackCommit          : DWORD;
    SizeOfHeapReserve          : DWORD;
    SizeOfHeapCommit           : DWORD;
    LoaderFlags                : DWORD;
    NumberOfRvaAndSizes        : DWORD;
    DataDirectory              : array [0..IMAGE_NUMBEROF_DIRECTORY_ENTRIES-1] of TImageDataDirectory;
  end;

  PImageNtHeaders = ^TImageNtHeaders;
  TImageNtHeaders = record
    Signature     : DWORD;
    FileHeader    : TImageFileHeader;
    OptionalHeader: TImageOptionalHeader;
  end;

  PImageSectionHeader = ^TImageSectionHeader;
  TImageSectionHeader = packed record
    Name                   : array [0..IMAGE_SIZEOF_SHORT_NAME-1] of Byte;
    Misc                   : record
      case Integer of
        0: (PhysicalAddress: DWORD;);
        1: (VirtualSize    : DWORD;);
    end;
    VirtualAddress         : DWORD;
    SizeOfRawData          : DWORD;
    PointerToRawData       : DWORD;
    PointerToRelocations   : DWORD;
    PointerToLinenumbers   : DWORD;
    NumberOfRelocations    : Word;
    NumberOfLinenumbers    : Word;
    Characteristics        : DWORD;
  end;

function ImageFirstSection(ntheader: PImageNtHeaders): PImageSectionHeader;

////////////////////////////////////////////////////////////////////////////////

function VirtualAllocEx(hProcess: HANDLE; lpAddress: LPVOID; dwSize: SIZE_T;
  flAllocationType: DWORD; flProtect: DWORD): LPVOID; stdcall;
function VirtualProtectEx(hProcess: HANDLE; lpAddress: LPVOID; dwSize: SIZE_T;
  flNewProtect: DWORD; var lpflOldProtect: DWORD): BOOL; stdcall;

function ZwUnmapViewOfSection(ProcessHandle: HANDLE;
  BaseAddress: PVOID): NTSTATUS; stdcall;

const
  ntdll = 'ntdll.dll';

implementation

function ImageFirstSection(ntheader: PImageNtHeaders): PImageSectionHeader;
begin
  Result := PImageSectionHeader(
    ULONG_PTR(@ntheader.OptionalHeader) +
    ntheader.FileHeader.SizeOfOptionalHeader);
end;


type
  TFNVirtualAllocEx = function(hProcess: HANDLE; lpAddress: LPVOID;
    dwSize: SIZE_T; flAllocationType: DWORD; flProtect: DWORD): LPVOID; stdcall;

var
  FNVirtualAllocEx: TFNVirtualAllocEx;

function VirtualAllocEx(hProcess: HANDLE; lpAddress: LPVOID; dwSize: SIZE_T;
  flAllocationType: DWORD; flProtect: DWORD): LPVOID; stdcall;
begin
  Result := nil;
  if not Assigned(FNVirtualAllocEx) then
    // assumes this image to be statically linked with kernel32.dll
    FNVirtualAllocEx := TFNVirtualAllocEx(
      GetProcAddress(GetModuleHandle(kernel32), 'VirtualAllocEx'));
  if not Assigned(FNVirtualAllocEx) then
    SetLastError(ERROR_CALL_NOT_IMPLEMENTED)
  else
    Result := FNVirtualAllocEx(hProcess, lpAddress, dwSize, flAllocationType,
      flProtect);
end;

type
  TFNVirtualProtectEx = function(hProcess: HANDLE; lpAddress: LPVOID;
    dwSize: SIZE_T; flNewProtect: DWORD; var lpflOldProtect: DWORD): BOOL;
    stdcall;

var
  FNVirtualProtectEx: TFNVirtualProtectEx;

function VirtualProtectEx(hProcess: HANDLE; lpAddress: LPVOID; dwSize: SIZE_T;
  flNewProtect: DWORD; var lpflOldProtect: DWORD): BOOL; stdcall;
begin
  Result := False;
  if not Assigned(FNVirtualProtectEx) then
    // assumes this image to be statically linked with kernel32.dll
    FNVirtualProtectEx := TFNVirtualProtectEx(
      GetProcAddress(GetModuleHandle(kernel32), 'VirtualProtectEx'));
  if not Assigned(FNVirtualProtectEx) then
    SetLastError(ERROR_CALL_NOT_IMPLEMENTED)
  else
    Result := FNVirtualProtectEx(hProcess, lpAddress, dwSize, flNewProtect,
      lpflOldProtect);
end;


type
  TFNZwUnmapViewOfSection = function(ProcessHandle: HANDLE;
  BaseAddress: PVOID): NTSTATUS; stdcall;

var
  FNZwUnmapViewOfSection: TFNZwUnmapViewOfSection;

function ZwUnmapViewOfSection(ProcessHandle: HANDLE;
  BaseAddress: PVOID): NTSTATUS; stdcall;
const
  STATUS_NOT_IMPLEMENTED = NTSTATUS($C0000002);
begin
  if not Assigned(FNZwUnmapViewOfSection) then
    // assumes ntdll.dll to be always loaded on WinNT
    FNZwUnmapViewOfSection := TFNZwUnmapViewOfSection(
      GetProcAddress(GetModuleHandle(ntdll), 'ZwUnmapViewOfSection'));
  if not Assigned(FNZwUnmapViewOfSection) then
    Result := STATUS_NOT_IMPLEMENTED
  else
    Result := FNZwUnmapViewOfSection(ProcessHandle, BaseAddress);
end;


end.
